/* ====================================================================
   Licensed to the Apache Software Foundation (ASF) under one or more
   contributor license agreements.  See the NOTICE file distributed with
   this work for additional information regarding copyright ownership.
   The ASF licenses this file to You under the Apache License, Version 2.0
   (the "License"); you may not use this file except in compliance with
   the License.  You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
==================================================================== */

package org.apache.poi.ss.formula.functions;

import junit.framework.TestCase;
import org.apache.poi.ss.formula.eval.ErrorEval;
import org.apache.poi.ss.formula.eval.StringEval;
import org.apache.poi.ss.formula.eval.ValueEval;

/**
 * Tests for {@link org.apache.poi.ss.formula.functions.Dec2Oct}
 *
 * @author cedric dot walter @ gmail dot com
 */
public final class TestDec2Oct extends TestCase {

    private static ValueEval invokeValue(String number1, String number2) {
        ValueEval[] args = new ValueEval[]{new StringEval(number1), new StringEval(number2),};
        return new Dec2Oct().evaluate(args, -1, -1);
    }

    private static ValueEval invokeValue(String number1) {
        ValueEval[] args = new ValueEval[]{new StringEval(number1),};
        return new Dec2Oct().evaluate(args, -1, -1);
    }

    private static void confirmValue(String msg, String number1, String number2, String expected) {
        ValueEval result = invokeValue(number1, number2);
        assertEquals(StringEval.class, result.getClass());
        assertEquals(msg, expected, ((StringEval) result).getStringValue());
    }

    private static void confirmValue(String msg, String number1, String expected) {
        ValueEval result = invokeValue(number1);
        assertEquals(StringEval.class, result.getClass());
        assertEquals(msg, expected, ((StringEval) result).getStringValue());
    }

    private static void confirmValueError(String msg, String number1, String number2, ErrorEval numError) {
        ValueEval result = invokeValue(number1, number2);
        assertEquals(ErrorEval.class, result.getClass());
        assertEquals(msg, numError, result);
    }

    public void testBasic() {
        confirmValue("Converts decimal 58 to octal (072)", "58", "3", "072");
        confirmValue("Converts decimal to octal (310)", "200", "3", "310");
        confirmValue("Converts decimal to octal (0000000310)", "200", "0000000310");

        confirmValue("Converts decimal to octal (7777777634)", "-100", "7777777634");

        confirmValue("Converts decimal 100 to hexadecimal with 10 (default) characters", "-100", "10", "7777777634");
        confirmValue("If argument places contains a decimal value, dec2hex ignores the numbers to the right side of the decimal point.", "58", "3.14159", "072");
    }

    public void testErrors() {
        //on number
        confirmValueError("Out of range min number", String.valueOf(-536870913 + -1), "0", ErrorEval.NUM_ERROR);
        confirmValueError("Out of range max number", String.valueOf(536870911 + 1), "0", ErrorEval.NUM_ERROR);

        //on places
        confirmValueError("072 requires more than places characters,", "58", "1", ErrorEval.NUM_ERROR);
        confirmValueError("If places is nonnumeric, this function returns the #VALUE! error value.", "ABCDEF", "0", ErrorEval.VALUE_INVALID);
        confirmValueError("If places is negative, this function returns the #NUM! error value.", "58", "-1", ErrorEval.NUM_ERROR);
    }
}
